package org.q.app.config;

import java.util.Map;

import org.q.toolkit.convert.construct.AbstractConstructor;
import org.q.toolkit.lang.Check;
import org.q.toolkit.lang.Contract;
import org.q.toolkit.lang.Str;
import org.q.toolkit.lang.collection.NewCollection;
import org.q.toolkit.lang.pattern.PatternTool;

/**
 * Convert the contents of an ini file to a java.util.Map and back.
 * 
 * @author Erik Lievaart
 */
@Contract
public class IniPropertiesConstructor extends AbstractConstructor<Map> {
	private static final char COMMENT_CHAR = '#';

	/**
	 * Create a string that can be written to an ini file.
	 * 
	 * @param map
	 *            key value pars of entries in the ini file. Keys are not allowed to contain the equals character '='.
	 * @return a Single string with all the contents of the Map in ini file format.
	 */
	public String constructString(final Map map) {
		StringBuilder builder = new StringBuilder();

		for (Object key : map.keySet()) {
			builder.append(key).append("=").append(map.get(key)).append('\n');
		}
		return builder.toString();
	}

	@Override
	public Map<String, String> constructObject(final String str) {
		Map<String, String> map = NewCollection.map();

		for (String entry : PatternTool.split("\n", str)) {
			if (Str.notBlank(entry) && entry.charAt(0) != COMMENT_CHAR) {
				String[] keyValue = entry.split("=", 2);
				String key = keyValue[0].trim();
				Check.isFalse(map.containsKey(key), "Duplicate key %", key);
				map.put(key, keyValue[1].trim());
			}
		}
		return map;
	}

	@Override
	public Class<Map> getLiteral() {
		return Map.class;
	}
}
