package org.q.app.engine.impl;

import java.util.Map;

import org.apache.commons.lang3.exception.ExceptionUtils;
import org.q.app.config.ResourcePaths;
import org.q.app.engine.script.ScriptConfig;
import org.q.app.engine.script.ScriptInvoker;
import org.q.toolkit.lang.collection.NewCollection;
import org.q.toolkit.swing.SwingThread;

import com.google.inject.Inject;

/**
 * Calls the Beanshell script for displaying error messages to the user.
 * 
 * @author Erik Lievaart
 */
public class ErrorScript {

	@Inject
	private ScriptInvoker runner;

	private ErrorScript() {
	}

	/**
	 * Invoke the error message script. This script will inform the user that there has been a problem.
	 * 
	 * @param message
	 *            Error message.
	 * @param thrown
	 *            The cause of the error.
	 */
	public void invoke(final String message, final Throwable thrown) {
		if (message == null) {
			thrown.printStackTrace();
			return;
		}
		SwingThread.invokeLater(new Runnable() {
			@Override
			public void run() {
				Map<String, Object> properties = NewCollection.map();
				properties.put("thrown", thrown);
				properties.put("cause", thrown.getCause() == null ? thrown : ExceptionUtils.getRootCause(thrown));
				properties.put("message", message);

				runner.createEngine(new ScriptConfig(properties));
				runner.runResource(ResourcePaths.UI_ERROR_SCRIPT);
				runner.eval("errorDialog(message, thrown, cause);");
			}
		});
	}
}