package org.q.app.engine.impl;

import org.q.app.contract.ValidationException;
import org.q.app.engine.concurrent.CommandScheduler;
import org.q.app.engine.parse.ShellCommand;
import org.q.app.engine.parse.ShellParseException;

import com.google.inject.Inject;
import com.google.inject.Provider;
import com.google.inject.Singleton;

/**
 * Schedules {@link ShellCommand}'s for execution.
 * 
 * @author Erik Lievaart
 */
@Singleton
class Plugins {

	@Inject
	private PluginMetadata metadata;
	@Inject
	private CommandScheduler scheduler;
	@Inject
	private Provider<PluginRunnable> callableProvider;

	private Plugins() {
	}

	/**
	 * Schedule the supplied ShellCommand for execution.
	 * 
	 * @param command
	 *            Command to schedule.
	 * @param raw
	 *            Original raw String invoked on the engine. Used for display reasons only.
	 * @throws ValidationException
	 *             Should never happen, but is possible if the command is scheduled without validating it first.
	 * @throws ShellParseException
	 *             Should never happen, but is possible if the command is scheduled without validating it first.
	 */
	void schedule(final ShellCommand command, final String raw) throws ValidationException, ShellParseException {
		metadata.validate(command);

		PluginRunnable runnable = callableProvider.get();
		runnable.setShellCommand(command);
		runnable.setLabel(raw);
		scheduler.schedule(runnable, metadata.getThreadPolicy(command));
	}

}