package org.q.app.engine.impl;

import java.util.Arrays;
import java.util.List;

import org.apache.commons.lang3.StringUtils;
import org.q.app.config.VariableConstants;
import org.q.app.engine.context.UserVariables;
import org.q.app.general.DetailMessage;
import org.q.toolkit.lang.Str;
import org.q.toolkit.lang.pattern.PatternTool;

import com.google.inject.Inject;

class SafeOperations {

	private static final List<String> UNSAFE = Arrays.asList("ant", "delete", "copy", "move");

	private final String message = "Disabled operations % to enable: >q.engine.setAllowUnsafeOperations(true);";
	private final DetailMessage error = DetailMessage.error(Str.sub(message, UNSAFE));

	@Inject
	private UserVariables variables;

	boolean isAllowed(final String raw) {
		return isAllowed(raw, StringUtils.equals(variables.getVariable(VariableConstants.ENGINE_UNSAFE), "true"));
	}

	static boolean isAllowed(final String raw, final boolean alwaysAllow) {
		String command = raw == null ? null : getCommand(raw);
		if (command == null || alwaysAllow) {
			return true;
		}
		for (String deny : UNSAFE) {
			if (deny.startsWith(command)) {
				return false;
			}
		}
		return true;
	}

	static String getCommand(final String raw) {
		return PatternTool.getGroup1("^([a-z]++).*+", raw.toLowerCase());
	}

	DetailMessage getError() {
		return error;
	}
}
