package org.q.app.engine.impl;

import org.q.app.engine.parse.ShellArgument;
import org.q.toolkit.lang.Str;
import org.q.toolkit.lang.concurrent.Prototype;

@Prototype
class ShellStringBuilder {

	private final StringBuilder builder;

	ShellStringBuilder() {
		builder = new StringBuilder();
	}

	ShellStringBuilder(final String name) {
		builder = new StringBuilder(name);
	}

	ShellStringBuilder appendFlagName(final String name) {
		builder.append(" -").append(name);
		return this;
	}

	ShellStringBuilder appendArgument(final ShellArgument argument) {
		builder.append(" ");

		switch (argument.getType()) {
			case BOOKMARK:
				builder.append('@').append(argument.getValue());
				break;

			case VARIABLE:
				builder.append('$').append(argument.getValue());
				break;

			case STRING:
				builder.append("`").append(argument.getValue()).append("`");
				break;

			default:
				throw new IllegalArgumentException(Str.sub("unknown type: %", argument.getType()));
		}
		return this;
	}

	ShellStringBuilder appendSwallowed(final String swallowed) {
		if (swallowed != null) {
			builder.append("|").append(swallowed);
		}
		return this;
	}

	@Override
	public String toString() {
		return builder.toString();
	}

}
