package org.q.app.engine.meta;

import java.util.List;

import org.apache.commons.lang3.builder.ToStringBuilder;
import org.q.app.contract.ThreadPolicy;
import org.q.toolkit.lang.collection.NewCollection;
import org.q.toolkit.lang.concurrent.RaceCondition;

@RaceCondition("Class is absolutely not thread safe!")
class CommandMetadataB {

	private final CommandNameMetadata name;
	private List<FlagGroupMetadata> groups = NewCollection.list();
	private boolean swallows = false;
	private boolean invocable = false;
	private boolean validate = false;
	private ThreadPolicy policy = ThreadPolicy.SWING;

	CommandMetadataB(final String qName, final String literal) {
		name = new CommandNameMetadata(qName, literal);
	}

	CommandNameMetadata getName() {
		return name;
	}

	CommandMetadataB setFlagGroupMetadata(final List<FlagGroupMetadata> groups) {
		this.groups = groups;
		return this;
	}

	CommandMetadataB setSwallows(final boolean swallows) {
		this.swallows = swallows;
		return this;
	}

	CommandMetadataB setInvocable(final boolean invocable) {
		this.invocable = invocable;
		return this;
	}

	void setValidate(final boolean validate) {
		this.validate = validate;
	}

	void setThreadPolicy(final ThreadPolicy threadPolicy) {
		policy = threadPolicy;
	}

	boolean isExecutable() {
		return invocable || swallows;
	}

	boolean isValidationLegal() {
		return swallows || !validate;
	}

	CommandMetadata create() {
		InvocationMetadata invocation = new InvocationMetadata(policy, invocable, swallows, validate);
		return new CommandMetadata(name, invocation, groups);
	}

	@Override
	public String toString() {
		return new ToStringBuilder(this).append(name.getQName()).append("class", name.getLiteral()).toString();
	}

	List<FlagGroupMetadata> getFlagGroupMetadata() {
		return groups;
	}

}
