package org.q.app.engine.meta;

import org.apache.commons.lang3.builder.ToStringBuilder;
import org.q.toolkit.lang.concurrent.Immutable;
import org.q.toolkit.reflect.LiteralTool;

/**
 * Meta data of the name of a plugin. This includes both the {@link Class} name and the plugin name. This class is
 * immutable.
 * 
 * @author Erik Lievaart
 */
@Immutable
public class CommandNameMetadata {

	private final String qName;
	private final String className;

	CommandNameMetadata(final String qName, final String literal) {
		this.qName = qName.toLowerCase();
		this.className = literal;
	}

	/**
	 * Get the name of the plugin.
	 */
	public String getQName() {
		return qName;
	}

	/**
	 * Get the {@link Class} defining the plugin.
	 */
	public Class<?> getLiteral() {
		return LiteralTool.getLiteral(className);
	}

	@Override
	public String toString() {
		return ToStringBuilder.reflectionToString(this);
	}

}
