package org.q.app.engine.script;

import org.q.app.config.VariableConstants;
import org.q.app.engine.context.UserVariables;
import org.q.app.engine.impl.Engine;
import org.q.toolkit.lang.Contract;

import com.google.inject.Inject;

/**
 * This class offers scrips access to the Q command line. Commands can be invoked literally or the engine templates can
 * be applied first.
 * 
 * @author Erik Lievaart
 */
@Contract
public class QEngineProxy implements Engine {

	@Inject
	@Core
	private Engine engine;

	@Inject
	@Templated
	private Engine templated;

	@Inject
	private UserVariables variables;

	QEngineProxy() {
	}

	@Override
	public void invoke(final String raw) {
		engine.invoke(raw);
	}

	/**
	 * Apply engine templates and invoke the command.
	 * 
	 * @param raw
	 *            Shell command to apply engine template for.
	 */
	public void invokeTemplated(final String raw) {
		templated.invoke(raw);
	}

	/**
	 * Set engine safe mode on or off. In safe mode the user cannot invoke the delete and move commands directly as
	 * ShellCommand from the command line. Thus the risk of accidentally deleting files from the command line is
	 * reduced, but certain file operations can only be performed from the UI or through scripting reducing the
	 * expressiveness of the command line.
	 * 
	 * @param flag
	 *            set to true to allow deleting and moving files from the command line, false to disable.
	 */
	public void setAllowUnsafeOperations(final Boolean flag) {
		variables.setPermanentVariable(VariableConstants.ENGINE_UNSAFE, flag.toString());
	}
}
