package org.q.app.general;

import com.google.inject.AbstractModule;
import com.google.inject.Scopes;
import com.google.inject.binder.AnnotatedBindingBuilder;
import com.google.inject.name.Named;
import com.google.inject.name.Names;

/**
 * Parent class for {@link com.google.inject.Module}'s with utility methods.
 * 
 * @author Erik Lievaart
 */
public abstract class ModuleBase extends AbstractModule {

	/**
	 * shorthand for calling Names.named(String name)
	 */
	protected Named name(final String name) {
		return Names.named(name);
	}

	/**
	 * Shorthand for binding a class as singleton.
	 * 
	 * @param <E>
	 *            Generic type of the class.
	 * @param clazz
	 *            literal to bind as singleton.
	 * @return the resulting {@link AnnotatedBindingBuilder}.
	 */
	protected <E> AnnotatedBindingBuilder<E> bindSingleton(final Class<E> clazz) {
		AnnotatedBindingBuilder<E> builder = bind(clazz);
		builder.in(Scopes.SINGLETON);
		return builder;
	}

}
