package org.q.app.general;

/**
 * Simple value holder for wrapping an Object with an optional label. This class' main use is to make an Object
 * available to the UI with a custom toString() method.
 * 
 * @author Erik Lievaart
 */
public class ValueHolder {

	private final Object content;
	private final String label;

	/**
	 * Constructor with the default toString().
	 * 
	 * @param content
	 *            contained object, also used for the toString() method.
	 */
	public ValueHolder(final Object content) {
		this.content = content;
		label = content.toString();
	}

	/**
	 * Constructor with custom label.
	 * 
	 * @param content
	 *            contained object.
	 * @param label
	 *            label to show the Object under.
	 */
	public ValueHolder(final Object content, final String label) {
		this.content = content;
		this.label = label;
	}

	/**
	 * Get the contained Object.
	 * 
	 * @return the contents.
	 */
	public Object getContent() {
		return content;
	}

	/**
	 * Returns the specified label.
	 * 
	 * @return the label
	 */
	@Override
	public String toString() {
		return label;
	}

}
