package org.q.app.ui.bind;

import java.awt.Component;
import java.util.Map;

import javax.swing.KeyStroke;

import org.q.app.config.ConfigurableResources;
import org.q.app.config.ResourcePaths;
import org.q.toolkit.convert.ConversionException;
import org.q.toolkit.convert.Converters;
import org.q.toolkit.lang.Check;
import org.q.toolkit.lang.collection.MultiMap;
import org.q.toolkit.lang.collection.NewCollection;

import com.google.inject.Inject;

class Bindings {

	@Inject
	private EngineActionLoader actions;

	private final MultiMap<String, Binding> bindings = NewCollection.multiMap();
	private final Converters converters;

	@Inject
	Bindings(final ConfigurableResources resources, final MenuConvertersProvider converters) {
		this.converters = converters.get();
		Map<String, String> ini = resources.loadIniFile(ResourcePaths.KEY_BINDINGS);

		for (String key : ini.keySet()) {
			BindingEntry entry = new BindingEntry(key, ini.get(key));
			bindings.add(entry.getComponentId(), createBinding(entry.getActionId(), entry.getBinding()));
		}
	}

	private Binding createBinding(final String actionId, final String binding) {
		if (binding.startsWith("$")) {
			return new ListenerBinding(actionId, binding);
		}
		try {
			return new KeyBinding(actionId, converters.to(KeyStroke.class, binding));
		} catch (ConversionException e) {
			throw new Error(e);
		}
	}

	void configure(final String componentId, final Component component) {
		Check.notNull(componentId, component);

		for (Binding binding : bindings.get(componentId)) {
			binding.register(component, actions.getAction(binding.getActionId()));
		}
	}

}
