package org.q.app.ui.bind;

import java.util.Map;

import javax.swing.Action;

import org.q.app.config.ConfigurableResources;
import org.q.app.config.ResourcePaths;
import org.q.app.config.ScriptResources;
import org.q.app.engine.impl.InputExaminer;
import org.q.app.general.DetailMessage;
import org.q.toolkit.io.RuntimeIOException;
import org.q.toolkit.lang.Check;
import org.q.toolkit.lang.Str;
import org.q.toolkit.lang.collection.NewCollection;

import com.google.inject.Inject;

class EngineActionLoader {
	private final static String TEMPLATE = "programming -r `%s`";

	private final Map<String, String> actions = NewCollection.map();

	@Inject
	private EngineActionFactory factory;

	@Inject
	void init(final ConfigurableResources resources, final ScriptResources scripts, final InputExaminer engine) {
		Map<String, String> ini = resources.loadIniFile(ResourcePaths.ACTIONS);

		for (String id : ini.keySet()) {
			DetailMessage examined = engine.examineRaw(resolveQCommand(scripts, ini.get(id)));
			if (examined.isError()) {
				throw new RuntimeIOException("Error in % (%) => %", ResourcePaths.ACTIONS, id, examined.getMessage());
			}
			actions.put(id, resolveQCommand(scripts, ini.get(id)));
		}
	}

	private String resolveQCommand(final ScriptResources resources, final String value) {
		if (!value.startsWith(">")) {
			return value;
		}
		for (String file : value.substring(1).split("\\s++")) {
			if (Str.notBlank(file) && !resources.isAvailable(file)) {
				throw new RuntimeIOException("Unable to read % referenced from %", file, ResourcePaths.ACTIONS);
			}
		}
		return String.format(TEMPLATE, value.substring(1));
	}

	Action getAction(final String id) {
		Check.isTrue(actions.containsKey(id), "No action with id % registered in %", id, ResourcePaths.ACTIONS);
		return factory.createEngineAction(actions.get(id));
	}
}
