package org.q.app.ui.bind;

import java.awt.event.ActionEvent;
import java.awt.event.FocusAdapter;
import java.awt.event.FocusEvent;
import java.awt.event.FocusListener;
import java.awt.event.KeyAdapter;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.awt.event.WindowListener;
import java.lang.reflect.InvocationHandler;
import java.lang.reflect.Method;
import java.util.EventObject;

import javax.swing.Action;

import org.q.toolkit.lang.Check;
import org.q.toolkit.reflect.LiteralTool;

class WrapAction {

	private WrapAction() {
	}

	private static ActionEvent dummyEvent(final EventObject e) {
		return new ActionEvent(e.getSource(), -1, WrapAction.class.getSimpleName() + ".dummy");
	}

	static KeyListener keyReleased(final Action action) {
		return new KeyAdapter() {
			@Override
			public void keyReleased(final KeyEvent e) {
				action.actionPerformed(dummyEvent(e));
			}
		};
	}

	// For param 2: MouseEvent.BUTTON1, MouseEvent.BUTTON2, MouseEvent.BUTTON3
	static MouseListener click(final Action action, final int button, final int clickCount) {
		return new MouseAdapter() {
			@Override
			public void mouseClicked(final MouseEvent e) {
				if (e.getButton() == button && e.getClickCount() == clickCount) {
					action.actionPerformed(dummyEvent(e));
				}
			}
		};
	}

	static FocusListener focusGained(final Action action) {
		return new FocusAdapter() {
			@Override
			public void focusGained(final FocusEvent arg0) {
				action.actionPerformed(dummyEvent(arg0));
			}
		};
	}

	static FocusListener focusLost(final Action action) {
		return new FocusAdapter() {
			@Override
			public void focusLost(final FocusEvent arg0) {
				action.actionPerformed(dummyEvent(arg0));
			}
		};
	}

	static WindowListener windowClosing(final Action action) {
		return new WindowAdapter() {
			@Override
			public void windowClosing(final WindowEvent e) {
				action.actionPerformed(dummyEvent(e));
			}
		};
	}

	@SuppressWarnings("unchecked")
	public static <E> E proxy(final Action action, final Class<E> clazz) {
		return (E) LiteralTool.createProxy(clazz, new InvocationHandler() {
			@Override
			public Object invoke(final Object proxy, final Method method, final Object[] args) {
				Check.isTrue(args.length > 0, "Method % has no arguments!", method);
				Check.isInstance(EventObject.class, args[0]);

				action.actionPerformed(dummyEvent((EventObject) args[0]));
				return null;
			}
		});
	}
}
