package org.q.app.vfs;

import java.util.TreeSet;

import org.apache.commons.lang3.StringUtils;
import org.q.toolkit.lang.Check;
import org.q.toolkit.lang.pattern.WildcardTool;

class WildcardResolver {

	private final TreeSet<String> files;
	private final String query;

	WildcardResolver(final String query, final TreeSet<String> fileNames) {
		Check.notNull(query, fileNames);
		this.files = fileNames;
		this.query = StringUtils.endsWith(query, "*") || StringUtils.endsWith(query, "?") ? query : query + "*";
	}

	String resolve() {
		String match = wildcardMatch();
		return match != null ? match : wildcardInsensitiveMatch();
	}

	String wildcardMatch() {
		for (String path : files) {
			if (WildcardTool.match(query, path)) {
				return path;
			}
		}
		return null;
	}

	String wildcardInsensitiveMatch() {
		for (String path : files) {
			if (WildcardTool.match(query.toLowerCase(), path.toLowerCase())) {
				return path;
			}
		}
		return null;
	}

}
