package org.q.plugins.join;

import java.io.BufferedInputStream;
import java.io.BufferedOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.util.List;

import org.q.app.contract.ValidationException;
import org.q.toolkit.io.UrlTool;
import org.q.toolkit.lang.Check;
import org.q.toolkit.lang.LogTemplate;

class FileJoiner {

	private final List<File> files;
	private final File result;
	private final LogTemplate log = new LogTemplate(getClass());

	public FileJoiner(final File dir, final List<File> join, final String destination) {
		this.files = join;

		result = new File(dir, UrlTool.unescape(destination));
	}

	public void invoke() throws IOException {
		Check.isFalse(result.exists(), "% already exists!", result);

		OutputStream out = new BufferedOutputStream(new FileOutputStream(result));
		for (File file : files) {
			log.debug("Appending % to %", file.getName(), result.getName());
			InputStream in = new BufferedInputStream(new FileInputStream(file));
			while (in.available() > 0) {
				out.write(in.read());
			}
			in.close();
		}
		out.close();
	}

	public void validate() throws ValidationException {
		ValidationException.unless(result.getParentFile().exists(), "Parent of % does not exist!", result);
		ValidationException.on(result.exists(), "Destination % exists!", result);
		ValidationException.on(files == null, "No files specified!");

		for (File file : files) {
			ValidationException.on(file == null || !file.exists(), "File % does not exist!", file);
			ValidationException.on(file.isDirectory(), "Cannot join directories", file);
		}
	}
}
