package org.q.plugins.join;

import java.io.File;
import java.util.List;

import org.q.app.contract.Invocable;
import org.q.app.contract.ThreadPolicy;
import org.q.app.contract.ValidationException;
import org.q.app.contract.annotation.Concurrency;
import org.q.app.contract.annotation.Doc;
import org.q.app.contract.annotation.Flag;
import org.q.app.ui.context.BrowseEvents;

import com.google.inject.Inject;

@Doc("Join files together. Files are joined in the order they were passed to the join command.")
@Concurrency(ThreadPolicy.QUEUE)
class JoinShellCommand implements Invocable {

	private FileJoiner join;

	@Inject
	private BrowseEvents events;

	@Flag(values = { "$urls", "$dir", "`joined.file`" }, primary = true)
	@Doc("Join files together. 1) files to join 2) dir to join in 3) file name to create")
	public JoinShellCommand urls(final List<File> files, final File dir, final String destination) {
		join = new FileJoiner(dir, files, destination);
		return this;
	}

	@Override
	public void invoke() throws Exception {
		join.invoke();
		events.refresh();
	}

	@Override
	public void validate() throws ValidationException {
		if (join != null) {
			join.validate();
		}
	}

}
