package org.q.plugins.various;

import static org.q.app.config.ResourcePaths.UI_POPUP_SCRIPT;
import static org.q.app.config.ResourcePaths.VARIOUS_MATCH_INIT_SCRIPT;
import static org.q.app.config.ResourcePaths.VARIOUS_MATCH_NEXT_SCRIPT;

import java.io.File;

import org.q.app.config.ResourcePaths;
import org.q.app.config.VariableConstants;
import org.q.app.contract.Invocable;
import org.q.app.contract.ThreadPolicy;
import org.q.app.contract.ValidationException;
import org.q.app.contract.annotation.Concurrency;
import org.q.app.contract.annotation.Doc;
import org.q.app.contract.annotation.Flag;
import org.q.app.engine.context.UserVariables;
import org.q.app.engine.script.ScriptConfig;
import org.q.app.engine.script.ScriptInvocation;
import org.q.app.engine.script.SwingScript;
import org.q.app.vfs.VirtualFile;

import com.google.inject.Inject;

@Doc("Collection of operations that don't belong anywhere else")
@Concurrency(ThreadPolicy.SINGLE)
class VariousShellCommand implements Invocable {

	private DirCalculator calculator;
	private FileMatcher matcher;
	private Mode mode;

	@Inject
	private SwingScript script;
	@Inject
	private UserVariables variables;

	private enum Mode {
		CALCULATE, MATCH;
	}

	@Flag(group = "main", values = "$dir")
	@Doc("Calculate the sizes of all nested children of a local directory")
	public VariousShellCommand calculate(final File dir) {
		this.calculator = new DirCalculator(dir);
		mode = Mode.CALCULATE;
		return this;
	}

	@Flag(group = "main", values = { "$dir", "$dir~" }, primary = true)
	@Doc("Match all files contained in the first argument to the directories contained in the second argument.")
	public VariousShellCommand match(final VirtualFile files, final VirtualFile dirs) {
		matcher = new FileMatcher(files, dirs);
		mode = Mode.MATCH;

		matcher.setDirSelector(variables.getVariable(VariableConstants.VARIOUS_MATCH_DIR_SELECTOR));
		matcher.setDirWords(variables.getVariable(VariableConstants.VARIOUS_MATCH_DIR_WORDS));
		matcher.setFileSelector(variables.getVariable(VariableConstants.VARIOUS_MATCH_FILE_SELECTOR));
		return this;
	}

	@Override
	public void validate() throws ValidationException {
	}

	@Override
	public void invoke() throws Exception {
		switch (mode) {
			case CALCULATE:
				ScriptInvocation invocation = new ScriptInvocation(ResourcePaths.UI_FORMAT);
				invocation.addResource(ResourcePaths.VARIOUS_CALCULATE_SCRIPT);
				script.invoke(new ScriptConfig(calculator.createProperties()), invocation);
				return;

			case MATCH:
				FileMatchResults.MATCHES.set(new FileMatchResults(matcher.match()));
				script.invoke(new ScriptConfig(matcher.createProperties()), new ScriptInvocation(UI_POPUP_SCRIPT,
						VARIOUS_MATCH_INIT_SCRIPT, VARIOUS_MATCH_NEXT_SCRIPT));
				return;
		}
		throw new IllegalStateException("Unknown constant: " + mode);
	}
}
