package org.q.toolkit.reflect.annotations;

import java.lang.annotation.Annotation;
import java.lang.reflect.Field;

import org.q.toolkit.lang.Check;
import org.q.toolkit.lang.concurrent.Immutable;
import org.q.toolkit.reflect.InstanceTool;
import org.q.toolkit.reflect.InstanceWrapper;

/**
 * Wrapper for a Field that has an Annotation.
 * 
 * @param <A>
 *            type of the annotation.
 * 
 * @author Erik Lievaart
 */
@Immutable
public class AnnotatedField<A extends Annotation> extends AnnotatedMember<Field, A> {

	AnnotatedField(final Field field, final A annotation) {
		super(field, annotation);
	}

	/**
	 * Inject a value into the annotated Field.
	 */
	public void inject(final Object target, final Object value) {
		Check.isInstance(getDeclaringClass(), target);
		InstanceWrapper wrapper = InstanceTool.wrap(target);
		wrapper.injectField(getName(), value);
	}

	public Object getValue(Object target) {
		Check.isInstance(getDeclaringClass(), target);
		InstanceWrapper wrapper = InstanceTool.wrap(target);
		return wrapper.getFieldValue(getMember().getName());
	}

	/**
	 * Get the type of the field.
	 */
	public Class<?> getType() {
		return getMember().getType();
	}

}
